local fs = require "nixio.fs" 

m = Map("iec60870-master", translate("IEC60870 Master"))

local connection_id
if arg[1] then
    connection_id = arg[1]
else
    luci.http.redirect(ds.build_url("admin/services/iec60870/iec60870_master"))
end

local cfgName
m.uci:foreach("iec60870-master", "iec60870_master", function(s)
    if s.connection_id == connection_id then
        cfgName = s[".name"]
    end
end)


section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
	translate("Enable IEC60870 Master"))
option_enabled.rmempty=false
option_enabled.default="1"

s = m:section(NamedSection, cfgName, "Slave device configuration", "Slave device configuration")
s.add_toggled = true

connection = s:option(ListValue, "connection", translate("Type"))
connection:value("rtu", translate("IEC60870-101 (RTU)"))
connection:value("tcp", translate("IEC60870-104 (TCP)"))
connection.default = "none"

id = s:option(Value, "id", "Id", translate("Id of IEC60870 Slave e.g Slave1"))
id.datatype = "string"

-- iec104 configuration --

host = s:option(Value, "host", "Host", translate("IEC60870 server's hostname or IP"))
host.datatype = "ip4addr"
host.default = "127.0.0.1"
host:depends("connection", "tcp")

port = s:option(Value, "port", "Port", translate("IEC60870 server's port number"))
port.datatype = "port"
port.default = "2404"
port:depends("connection", "tcp")

k = s:option(Value, "k", "K", translate("Maximum unacknowledged transmitted APDUs"))
k.datatype = "integer"
k.default = "12"
k:depends("connection", "tcp")

w = s:option(Value, "w", "W", translate("Maximum unacknowledged received APDUs"))
w.datatype = "integer"
w.default = "8"
w:depends("connection", "tcp")

t0 = s:option(Value, "t0", "T0", translate("Amount of time that the IEC60870 Master will attempt to connect to the device"))
t0.datatype = "range(1,30)"
t0.default = "2"
t0:depends("connection", "tcp")

t1 = s:option(Value, "t1", "T1", translate("Time to wait for a acknowledgment (“ACK”) to a transmitted APDU"))
t1.datatype = "integer"
t1.default = "15"
t1:depends("connection", "tcp")

t2 = s:option(Value, "t2", "T2", translate("Time to wait before sending a supervisory APDU ACK"))
t2.datatype = "integer"
t2.default = "10"
t2:depends("connection", "tcp")

t3 = s:option(Value, "t3", "T3", translate("Idle time before sending TEST APDU"))
t3.datatype = "integer"
t3.default = "20"
t3:depends("connection", "tcp")

-- iec101 configuration --

serial_port = s:option(Value, "serial_port", translate("Serial Port"))
serial_port.default = "/dev/ttyS0"
serial_port:depends("connection", "rtu")

speed = s:option(ListValue, "baud", translate("BaudRate"))
speed:value("300", translate("300"))
speed:value("600", translate("600"))
speed:value("1200", translate("1200"))
speed:value("2400", translate("2400"))
speed:value("4800", translate("4800"))
speed:value("9600", translate("9600"))
speed:value("19200", translate("19200"))
speed:value("38400", translate("38400"))
speed:value("19200", translate("19200"))
speed:value("57600", translate("57600"))
speed.default = "9600"
speed:depends("connection", "rtu")

parity = s:option(ListValue, "parity", translate("Parity"))
parity:value("N", translate("None"))
parity:value("E", translate("Even"))
parity:value("O", translate("Odd"))
parity.default = "E"
parity:depends("connection", "rtu")

databits = s:option(ListValue, "data_bit", translate("Data bits"))
databits:value("6", "6")
databits:value("7", "7")
databits:value("8", "8")
databits.default = "8"
databits:depends("connection", "rtu")

stopbits = s:option(ListValue, "stop_bit", translate("Stop bits"))
stopbits:value("1", "1")
stopbits:value("2", "2")
stopbits.default = "1"
stopbits:depends("connection", "rtu")

operation_mode = s:option(ListValue,"operation_mode",translate("Link Transmission Procedure"))
operation_mode:value("unbalanced","Unbalanced")
operation_mode:value("balanced","Balanced")
operation_mode.default = "balanced"
operation_mode:depends("connection", "rtu")

iec101_master_addr = s:option(Value,"iec101_master_addr","Master Address",translate("Set the own link layer or Master address (only balanced mode)"))
iec101_master_addr.datatype    = "and(min(1),uinteger)"
iec101_master_addr.placeholder = "0-65535"
iec101_master_addr.default = "1"
iec101_master_addr:depends("operation_mode", "balanced")
iec101_master_addr:depends("connection", "rtu")

iec101_slave_addr = s:option(Value,"iec101_slave_addr","Slave Address",translate("Link Address or Slave Address"))
iec101_slave_addr.datatype    = "and(min(0),uinteger)"
iec101_slave_addr.placeholder = "0-65535"
iec101_slave_addr.default = "1"
iec101_slave_addr:depends("connection", "rtu")

addressLength = s:option(ListValue,"addressLength",translate("Address Length"))
addressLength:value("1","One Octet")
addressLength:value("2","Two Octets")
addressLength.default = "1"
addressLength:depends("connection", "rtu")

timeoutRepeat = s:option(Value, "timeoutRepeat", "Timeout Repeat", translate("Timeout for repeated message transmission when no ACK received in milliseconds"))
timeoutRepeat.datatype = "integer"
timeoutRepeat.default = "1000"
timeoutRepeat:depends("connection", "rtu")

timeoutForAck = s:option(Value, "timeoutForAck", "Timeout For Ack", translate("Timeout for link layer ACK in milliseconds "))
timeoutForAck.datatype = "integer"
timeoutForAck.default = "200"
timeoutForAck:depends("connection", "rtu")

useSingleCharACK = s:option(ListValue, "useSingleCharACK", "Use Single Char ACK", translate("Use single char ACK or RESP_NO_USER_DATA "))
useSingleCharACK:value("0", "No")
useSingleCharACK:value("1", "Yes")
useSingleCharACK.default = "0"
useSingleCharACK:depends("connection", "rtu")

-- common configuration --

sizeof_cot = s:option(ListValue, "sizeof_cot", "Size Of COT", translate("Specify the number of octets in an ASDU COT field"))
sizeof_cot:value("1", translate("1"))
sizeof_cot:value("2", translate("2"))
sizeof_cot.default = "2"

oa = s:option(Value, "oa", "OA", translate("This is required when there is more than one controlled station(master/client)"))
oa.datatype = "range(0,254)"
oa.default = "0"

ca = s:option(Value, "ca", "CA", translate("Common Address or ASDU Address"))
ca.datatype = "range(0,254)"
ca.default = "1"

sizeof_ca = s:option(ListValue, "sizeof_ca", "Size Of CA", translate("Size Of Common Address or ASDU Address for IEC101 is 1 and for  IEC104 is 2"))
sizeof_ca:value("1", translate("1"))
sizeof_ca:value("2", translate("2"))
sizeof_ca.default = "2"

sizeof_ioa = s:option(ListValue, "sizeof_ioa", "Size Of IOA", translate("Size Of Information Object Address"))
sizeof_ioa:value("1", translate("1"))
sizeof_ioa:value("2", translate("2"))
sizeof_ioa:value("3", translate("3"))
sizeof_ioa.default = "3"

max_asdu_length = s:option(ListValue, "max_asdu_length", "Max. ASDU length", translate("Maximum size of ASDU that is generated,Its value is 249 for IEC104 and 254 for IEC101."))
max_asdu_length:value("249", translate("249"))
max_asdu_length:value("254", translate("254"))
max_asdu_length.default = "249"


gi_interval = s:option(Value, "gi_interval", "GI Time Interval (Seconds)", translate("General Interrogation based on a specified time interval"))
gi_interval.datatype = "integer"


cs_interval = s:option(Value, "cs_interval", "CS Time Interval (Seconds)", translate("if and when a time synchronization should be sent to the slave"))
cs_interval.datatype = "integer"

return m

